import * as React from 'react';
import {useEffect, useRef} from 'react';
import {observer} from "mobx-react-lite";
import {
    Conditional,
    FormProps,
    IfFalse,
    IfTrue,
    ModelCheckbox,
    ModelDateField,
    ModelForm,
    ModelInput,
    PaddedPaper
} from "@rx-suite/ui";
import {t, Trans} from '@lingui/macro';
import Typography from '@material-ui/core/Typography';
import {StyleTypeSelect} from './fields/StyleTypeSelect';
import {StylesSelect} from "./fields/StylesSelect";
import {Container, Grid, makeStyles} from '@material-ui/core';
import {StyleMaterialSelect} from "./fields/StyleMaterialSelect";
import {GeneratedFields} from "./fields/GeneratedFields";
import {JobTypeSelect} from './fields/JobTypeSelect';
import {FrameStatusSelect} from './fields/FrameStatusSelect';
import {AddonSelect} from './fields/AddonSelect';
import {ShippingMethodSelect} from "./fields/ShippingMethodSelect";
import {useOrderEntryStore} from "../../hooks/useOrderEntryStore";
import {OrderModel} from "../../stores/OrderStore";
import {CurrentUserConfig} from '../../stores/ConfigStore';
import {QuickCodeSelect} from "./fields/QuickCodeSelect";
import {ShippingFields} from "./fields/ShippingFields";
import {CustomerNameField} from "./fields/CustomerNameField";
import {QuickCodeStore} from "../../stores/QuickCodeStore";
import {useCustomerSelectionStore} from "../../hooks/useCustomerSelectionStore";
import {routerStore} from "@rx-suite/data";
import {CustomerOrderSelect} from "./fields/CustomerOrderSelect";
import {red} from "@material-ui/core/colors";
import {WantedDateField} from "./fields/WantedDateField";
import {LabMaintenanceStore} from "../../stores/LabMaintenanceStore";
import {PurchaseOrderInputField} from "./fields/PurchaseOrderInputField";
import {ReleaseNumberInputField} from "./fields/ReleaseNumberInputField";

const useStyles = makeStyles((theme) => ({
    section: {
        marginTop: theme.spacing(4)
    },
    error: {
        color: red[500]
    }
}));

export const OrderForm: React.FunctionComponent<FormProps> = observer(({model, readOnly}) => {
    const store = useOrderEntryStore();
    const currentCustomerStore = useCustomerSelectionStore();
    const currentUserConfig = CurrentUserConfig.get();
    const classes = useStyles();

    const formRef = useRef<HTMLFormElement>(null);

    useEffect(() => {
        store.maybeGetFrameFinderOptions();
    }, []);

    useEffect(() => {
        console.log("Setting target");
        store.setTarget(model as OrderModel, readOnly);
    }, [store, model, readOnly, currentUserConfig]);

    const handleRedirect = React.useCallback((e, resp) => {
        if(resp) {
            if(currentUserConfig?.continuous_entry) {
                routerStore.history.push(model.store.createLink())
            } else {
                routerStore.history.push(model.store.listLink())
            }
        }
    }, [model, currentUserConfig])

    React.useEffect(() => {
        const hotkey = (e: any) => {
            if (e.ctrlKey && e.key === 's') {
                e.stopImmediatePropagation();
                e.preventDefault();
                if (model.viewModel.isValid && !readOnly) {
                    formRef.current?.requestSubmit();
                }
            }
        }
        document.addEventListener('keydown', hotkey)
        return (() => {
            document.removeEventListener('keydown', hotkey)
        })
    }, [model.viewModel.isValid, readOnly])

    return (
        <Container maxWidth={"xl"}>
            <PaddedPaper>
                <ModelForm model={model} formRef={formRef} preventSubmitOnEnter afterSubmit={handleRedirect} suppressWrapper suppressImportExport readOnly={readOnly || currentCustomerStore.noCustomers} >
                    <Typography variant={'h4'}>{t`General Information`}</Typography>
                    {model.viewModel.isInvalidJob && <>
                        <Typography variant={'subtitle2'} className={classes.error}>
                            <Conditional condition={(model as OrderModel).viewModel.isFrameOnlyJob}>
                                <IfTrue>
                                    <Trans>The order has error: Frame not found in Rx-Inventory</Trans>
                                </IfTrue>
                                <IfFalse>
                                    <Trans>The order has a calculation error in Rx-Universe: {model.viewModel.unvalid_message}</Trans>
                                </IfFalse>
                            </Conditional>
                        </Typography>
                    </>
                    }
                    <Grid container spacing={4}>
                        {!!model.viewModel.order_number && <Grid item xs={12}>
                            <ModelInput name={"order_number"} disabled/>
                        </Grid>}
                        <Grid item xs={12}>
                            <Grid container spacing={4}>
                                <Grid item xs={12} lg={4}>
                                    <CustomerNameField/>
                                </Grid>
                                <Grid item xs={12} lg={4}>
                                    <CustomerOrderSelect/>
                                </Grid>
                                <Grid item xs={12} lg={4}>
                                    <ModelDateField disabled name={"date_ordered"} timeFormat={false}/>
                                </Grid>
                                <WantedDateField />
                                <Grid item xs={12} lg={4}>
                                    <JobTypeSelect/>
                                </Grid>
                                <Grid item xs={12} lg={4}>
                                    <FrameStatusSelect/>
                                </Grid>
                                <Grid item xs={12} lg={4}>
                                    <PurchaseOrderInputField/>
                                </Grid>
                                <Grid item xs={12} lg={4}>
                                    <ReleaseNumberInputField />
                                </Grid>
                                {
                                    readOnly &&
                                    <>
                                        <Grid item xs={12} lg={4}>
                                            <ModelInput name={"submitted_by"}
                                                        hint={{title: 'Submitted By', text: 'The User that submitted this order.'}}
                                                        disabled
                                            />
                                        </Grid>
                                        <Grid item xs={12} lg={4}>
                                            <ModelDateField disabled name={"date_completed"} timeFormat={false}/>
                                        </Grid>
                                    </>
                                }
                                {
                                    LabMaintenanceStore.selectedModel?.display_hold_switch && <Grid item xs={12} lg={4}>
                                        <ModelCheckbox name={"hold_at_lab"} />
                                    </Grid>
                                }
                            </Grid>
                        </Grid>
                    </Grid>

                    <Grid container className={classes.section}>
                        <Grid item xs={12}>
                            <Typography variant={"h6"}>{t`Shipping Information`}</Typography>
                        </Grid>
                        <Grid item xs={12}>
                            <Grid container spacing={2}>
                                <Grid item xs={12} lg={4}>
                                    <ShippingMethodSelect/>
                                </Grid>
                                <ShippingFields/>
                            </Grid>
                        </Grid>
                    </Grid>

                    <Grid container className={classes.section}>
                        {!model.isFrameOnlyJob && <>
                        <Grid item xs={12}>
                            <Typography variant={"h6"}>{t`Lens Information`}</Typography>
                        </Grid>
                        </>}
                        <Grid item xs={12}>
                            {!model.isAddonJob && !model.isFrameOnlyJob && <>
                                {!!QuickCodeStore.models.length && <Grid container spacing={4}>
                                    <Grid item xs={12} lg={4}>
                                        <QuickCodeSelect/>
                                    </Grid>
                                </Grid>}
                                <Grid container spacing={4}>
                                    <Grid item xs={12} lg={4}>
                                        <StyleTypeSelect/>
                                    </Grid>
                                </Grid>
                            </>}
                            <Grid container spacing={4}>
                                {!model.isAddonJob && !model.isFrameOnlyJob &&  <>
                                    <Grid item xs={12} lg={4}>
                                        <StylesSelect/>
                                    </Grid>
                                    <Grid item xs={12} lg={4}>
                                        <StyleMaterialSelect/>
                                    </Grid>
                                </>}
                                {!model.isFrameOnlyJob &&  <>
                                    <Grid item xs={12} lg={4}>
                                        <AddonSelect/>
                                    </Grid>
                                </>}
                                <Grid item xs={12}>
                                    <GeneratedFields/>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Grid>
                    {
                        (store.hasFields || model.isAddonJob) &&
                        <Grid container className={classes.section} spacing={2}>
                            <Grid item xs={12}>
                                <Typography variant={"h6"}>{t`Comments/Remarks`}</Typography>
                            </Grid>
                            <Grid item xs={12}>
                                <ModelInput name={'comments'} multiline minRows={5} variant={'outlined'}/>
                            </Grid>
                        </Grid>
                    }
                </ModelForm>
            </PaddedPaper>
        </Container>
    )
});
