import * as React from 'react';
import { observer } from 'mobx-react-lite';
import {
    AsyncModelSelect,
    communicateWithModule,
    Conditional,
    HintButton,
    IfFalse,
    IfTrue, ModelInput,
    SelectFileField, useModelContext
} from "@rx-suite/ui";
import {OrderFieldProps, sortedOptions} from "../../../stores/_FieldConstants";
import {SelectOption} from "react-select-material-ui";
import debounceAsync from 'debounce-async';
import {Grid, Typography} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import {useOrderEntryStore} from "../../../hooks/useOrderEntryStore";

const fn = async (query: string, prevOptions: SelectOption[], additional: any) => {
    const response = await communicateWithModule("rxuniverse_api", "/api/frames/traces", {
        "first-key": query,
        limit: 10,
        offset: additional.page * 10
    });
    const newOptions: SelectOption[] = [];

    response.models.forEach((o: { name: string }) => {
        const exists = prevOptions.some(p => p.value === o.name);
        if(!exists) {
            newOptions.push({ value: o.name, label: o.name as string })
        }
    })

    return({
        options: sortedOptions(newOptions),
        hasMore: response.count > prevOptions.length,
        additional: {
            ...additional,
            page: additional.page + 1
        }
    })
}

export const FrameTraceFields: React.FunctionComponent<OrderFieldProps> = observer(({name, label}) => {
    const store = useOrderEntryStore();
    const loading = store.loadingFrameTraceFile;
    const file = store.frameTraceFile;
    const traceId = store.order?.viewModel?.dig(name)
    const loadOptions = React.useCallback(debounceAsync(fn, 320), [])
    const readOnly = store.lockFrameFields;
    const model = useModelContext();

    React.useEffect(() => {
        store.handleTraceIdChange()
    }, [model.viewModel?.dig(name)])

    return(
            <Conditional condition={readOnly}>
                <IfTrue>
                    <ModelInput name={name} label={traceId ? label : t`No trace used`}></ModelInput>
                </IfTrue>
                <IfFalse>
                    <Grid container direction="row" >
                        <Grid container item>
                            <AsyncModelSelect
                                name={name}
                                label={t`Frame Trace`}
                                loadOptions={loadOptions}
                                clearable
                                fullWidth
                            />
                        </Grid>
                        <Grid container item xs={12}>
                            <Typography><Trans>Or upload a trace file</Trans></Typography>
                        </Grid>
                        <Grid container item xs={"auto"}>
                            <SelectFileField
                                label={file ? `${t`File:`} ${file.name  || t`Unnamed`}` : t`Choose file`}
                                onDrop={store.handleDrop}
                                disabled={loading}
                                buttonProps={{loading: loading, color: file ? 'primary' : 'secondary'}}/>
                            <HintButton title={t`File Select`}
                                        text={t`Upload a file with a supported trace format.`}
                                        boxProps={{style: {display: 'inline-block'}}}/>
                        </Grid>
                    </Grid>
                </IfFalse>
            </Conditional>
    )
})